#include "game.h"
#include <vgagl.h>
#include <stdio.h>

#define VideoMode G640x480x16
#define NCubes 196

GraphicsContext *physicalscreen;
GraphicsContext *virtualscreen;

int
main ()
{
  int key, done, autoplay;
  done = autoplay = 0;

  vga_init ();
  vga_setmode (VideoMode);
  int width = vga_getxdim ();
  int height = vga_getydim ();
  gl_setcontextvga (VideoMode);

  physicalscreen = gl_allocatecontext ();
  gl_getcontext (physicalscreen);
  gl_setcontextvgavirtual (VideoMode);
  virtualscreen = gl_allocatecontext ();
  gl_getcontext (virtualscreen);

  gl_setcontext (virtualscreen);
  gl_clearscreen (BGcolor);

  TGame game(width, height);
  TPlatform plato (width, height, &game);
  TBall ball (width, height, &plato);

  TCube cubes[NCubes];
  for (int i = 0; i < NCubes; i++)
    {
      cubes[i].x = (i % 49) * 13 + 2;
      cubes[i].y = (i / 49) * 13 + 20;
      cubes[i].w = 12;
      cubes[i].h = 12;
      cubes[i].sethit ((i / 49) + 1);
    }

  gl_setfont (8, 8, gl_font8x8);
  gl_setwritemode (FONT_COMPRESSED + WRITEMODE_OVERWRITE);
  gl_setfontcolors (BGcolor, vga_white ());
  game.update ();

  while (!done)
    {
      key = vga_getkey ();
      if (key == 27)
	{
	  key = vga_getkey ();
	  if (key == '[')
	    {
	      key = vga_getkey ();
	      switch (key)
		{
		case 'D':
		  plato.move (0);
		  break;
		case 'C':
		  plato.move (1);
		  break;
		case 'B':
		  autoplay = 1 - autoplay;
		}
	    } else done = 1;
	}

      // Auto mode
      if (autoplay)
	{
	  if (plato.x > ball.x)
	    plato.move (0);
	  else if (plato.x + plato.w < ball.x)
	    plato.move (1);
	}
      // Check for ball out
      if (!ball.move ())
      {
        puts("Ball is out");
	break;
      }

      for (int i = 0; i < NCubes; i++)
	{
	  if (cubes[i].visible && ball.touchcube (&cubes[i]))
	    {
	      puts ("Kicked cube");
	      game.addscore (10 * cubes[i].gethit ());
	      cubes[i].sethit (cubes[i].gethit () - 1);
	      ball.ay *= -1;
	      beep();
	    }
	}
      game.play ();
      gl_copyscreen (physicalscreen);
    }
  if (done)
   puts("All done");
  vga_setmode (TEXT);
  return 0;
}
