#include "game.h"

void
beep()
{
 //
}

// TGame
TGame::TGame (int wd, int hd)
{
  width = wd;
  height = hd;
  scores = 0;
  lastmove = 0;
  gmcycle = 0;
}

void
TGame::play ()
{
  gmcycle++;
}

void
TGame::moved ()
{
  lastmove = gmcycle;
}

char
TGame::freshmove ()
{
  return char (gmcycle - lastmove < 5);
}

void
TGame::update ()
{
  char s[20];
  sprintf (s, "Score: %d", scores);
  gl_write (5, height - 12, s);
}

void
TGame::addscore (int s)
{
  if (s > 0)
    {
      scores += s;
      update ();
    }
}

void
TGame::setscore (int s)
{
  if (s > 0)
    {
      scores = s;
      update ();
    }
}

// TBall

TBall::TBall (int wd, int hd, TPlatform * p)
{
  rad = 5;
  x = wd / 2; y = hd / 2;
  ax = 1; ay = -1;
  width = wd; height = hd;
  pl = p;
  show ();
}

void
TBall::draw (int col)
{
  gl_circle (int (x), int (y), rad, col);
}

void
TBall::show ()
{
  draw (FRcolor);
}

void
TBall::hide ()
{
  draw (BGcolor);
}

char
TBall::touchplato (TPlatform * p)
{
  return ((x >= p->x) && (x <= p->x + p->w) && (y + rad >= p->y)) ? 1 : 0;
}

char
TBall::touchcube (TCube * c)
{
  return ((x >= c->x) && (x <= c->x + c->w) && (y + rad >= c->y)
	  && (y - rad <= c->h + c->y)) ? 1 : 0;
}

char
TBall::move ()
{
  hide ();
  // MOVE
  x += ax;
  y += ay;

  // Side walls
  if ((x + rad >= width) || (x - rad <= 0))
    {
      ax *= -1;
      x = (x - rad <= 0) ? rad : width - rad;
      if (abs(int(ax)) > 0.3)
       ax += -0.3 * ((ax > 0) * 2 - 1);
    }

  // Upper wall
  if (y <= rad)
    {
      ay *= -1;
      y = rad;
    }

  if (y + rad > pl->y)
    {
      // Special code here
      y -= 2*rad;
      return 0;
    }

  if (touchplato (pl))
    {
      ay *= -1;
      y = pl->y - rad;
      ax += 0.1 * ((pl->gm->freshmove () * -2) - 1) * (int(ax > 0) * 2 - 1);
    }
  show ();
  return 1;
}

// TCube

TCube::TCube ()
{
  visible = 1;
  hitcount = 1;
}

void
TCube::draw (int col)
{
  gl_fillbox (x, y, w, h, col);
}

void
TCube::show ()
{
  if (visible)
    draw (cl);
}

void
TCube::hide ()
{
  draw (BGcolor);
}

void
TCube::sethit (int val)
{
  if (val < 5)
    {
      if (val > 0)
	{
	  hitcount = val;
	  cl = 16 - hitcount;
	  show ();
	}
      else
	{
	  visible = 0;
	  hitcount = 0;
	  cl = 7;
	  hide ();
	}
    }
}

int
TCube::gethit ()
{
  return hitcount;
}

// TPlatform

void
TPlatform::draw (int col)
{
  gl_fillbox (x, y, w, h, col);
}

void
TPlatform::show ()
{
  draw (cl);
}

void
TPlatform::hide ()
{
  draw (BGcolor);
}

TPlatform::TPlatform (int wd, int hd, TGame * g)
{
  w = 64;
  h = 7;
  xdelta = w2;
  cl = FRcolor2;
  x = wd / 2;
  y = hd - h - 15;
  width = wd;
  height = hd;
  gm = g;
  show ();
}

void
TPlatform::move (int dir)
{
  hide ();
  switch (dir)
    {
    case 0:
      x -= xdelta;
      if (x <= 0)
	x = 0;
      break;
    case 1:
      x += xdelta;
      if (x + w >= width)
	x = width - w;
    }
  gm->moved ();
  show ();
}
